// game.h
//  Game GameState

#include "tinparty.h"

using namespace std;

Block::Block(): min(1), max(10), rolling(true)
{
}

EStatus Block::Tick(double dtime)
{
	if(rolling)
	{
		progress += dtime;
		if(progress >= .1)
		{
			value = (uint32_t) (((double) rand() / RAND_MAX) * (max - min + 1)) + min;
		}
		progress = fmod(progress, .1);
	}
	return S_Continue;
}

void Block::Draw(BITMAP* dest, int x, int y, int w, int h)
{
	rectfill(dest, x, y, x + w - 1, y + h - 1, makecol(255, 255, 255));
	rect(dest, x, y, x + w - 1, y + h - 1, 0);
	textprintf_centre(dest, font, x + (w / 2), y + (h / 2) - (text_height(font) / 2), 0, "%d", value);
}
		
void Block::Hit()
{
	rolling = false;
}

HUD::HUD(): player(NULL), num(0)
{
}

EStatus HUD::Tick(double dtime)
{
	return S_Continue;
}

void HUD::Draw(BITMAP* dest, int x, int y, int w, int h)
{
	player->instance->DrawPortrait(dest, x, y, w, h);
	textprintf(dest, font, x, y, makecol(255, 255, 255), "%d", num);
}

TurnOrderFinder::~TurnOrderFinder()
{
	delete textbox;
	textbox = NULL;
	for(vector<Block*>::iterator i = blocks.begin(); i != blocks.end(); i++)
		delete *i;
}

bool TurnOrderFinder::InitLogic()
{
	textbox = new SubStateText("It's a race! Try to get farther than anyone else! Ready to play? Now we just have to figure out our turn order! Everybody jump!", 10, 100, SCREEN_W - 158, 128, TM_Filled);
	
	return textbox;
}

bool TurnOrderFinder::InitGraphics()
{
	int p = -2;
	for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++, p++)
	{
		(*i)->Move(-SCREEN_W / 2 - 64, 0);
		(*i)->MoveTo(p * 94, 150, 3);
		(*i)->GotoState(PS_Rolling);
	}
	
	return true;
}

EStatus TurnOrderFinder::Tick(double dtime)
{
	GGame.board->Tick(dtime);
	uint32_t p = 0;
	for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++, p++)
	{
		switch((*i)->Tick(dtime))
		{
			case S_Notice:
				for(vector<Block*>::iterator j = blocks.begin(); j != blocks.end(); j++)
					if((*j)->pattachment == p)
					{
						++ndone;
						(*j)->Hit();
						break;
					}
				break;
			case S_Next:
			{
				if(ndone2 == 5)
				{
					RequestState(MainGame::Factory);
					return S_Next;
				}
				if((*i)->state == PS_Rolling)
				{
					Block* b = new Block;
					b->pattachment = p;
					blocks.push_back(b);
				}
				break;
			}
		}
	}
	for(vector<Block*>::iterator i = blocks.begin(); i != blocks.end(); i++)
		(*i)->Tick(dtime);
	
	if(ndone2 == 4)
	{
		progress += dtime;
		if(progress  >=.5 && textbox->Tick(dtime) == S_Next)
		{
			int x, y;
			for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
			{
				GGame.board->GetLocationOfSquare(0, x, y);
				(*i)->player->location = 0;
				(*i)->player->MoveTo(x, y, 3);
			}
			for(vector<Block*>::iterator j = blocks.begin(); j != blocks.end(); j++)
				delete *j;
			blocks.clear();
			++ndone2;
		}
	}
	else if(ndone2 == 5)
	{
		camx = GGame.huds[0]->player->x;
		camy = GGame.huds[0]->player->y;
	}
	else if(ndone == GGame.players.size())
	{
		if(!counting && textbox->Tick(dtime) == S_Next)
		{
			progress = 0;
			counting = true;
		}
		if(counting)
			progress += dtime;
		if(ndone2 == 0)
		{
			HUD* h = new HUD;
			h->num = 1;
			Pawn* min = NULL;
			int minn = 0;
			for(vector<Block*>::iterator i = blocks.begin(); i != blocks.end(); i++)
			{
				if((*i)->value > minn)
				{
					minn = (*i)->value;
					min = GGame.players[(*i)->pattachment];
				}
			}
			delete textbox;
			msgtext.str("");
			msgtext << "Player " << min->pnum << " goes first!";
			textbox = new SubStateText(msgtext.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			counting = false;
			progress = 0;
			h->player = min;
			GGame.huds.push_back(h);
			++ndone2;
		}
		if(progress >= .5 && ndone2 == 1)
		{
			HUD* h = new HUD;
			h->num = 2;
			Pawn* min = NULL;
			int minn = 0;
			for(vector<Block*>::iterator i = blocks.begin(); i != blocks.end(); i++)
			{
				if((*i)->value > minn && GGame.players[(*i)->pattachment] != GGame.huds[0]->player)
				{
					minn = (*i)->value;
					min = GGame.players[(*i)->pattachment];
				}
			}
			delete textbox;
			msgtext.str("");
			msgtext << "Player " << min->pnum << " goes next!";
			textbox = new SubStateText(msgtext.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			counting = false;
			progress = 0;
			h->player = min;
			GGame.huds.push_back(h);
			++ndone2;
		}
		if(progress >= .5 && ndone2 == 2)
		{
			HUD* h = new HUD;
			h->num = 3;
			Pawn* min = NULL;
			int minn = 0;
			for(vector<Block*>::iterator i = blocks.begin(); i != blocks.end(); i++)
			{
				if((*i)->value > minn && GGame.players[(*i)->pattachment] != GGame.huds[0]->player && GGame.players[(*i)->pattachment] != GGame.huds[1]->player)
				{
					minn = (*i)->value;
					min = GGame.players[(*i)->pattachment];
				}
			}
			delete textbox;
			msgtext.str("");
			msgtext << "Then player " << min->pnum << "...";
			textbox = new SubStateText(msgtext.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			h->player = min;
			counting = false;
			progress = 0;
			GGame.huds.push_back(h);
			++ndone2;
		}
		if(progress >= .5 && ndone2 == 3)
		{
			HUD* h = new HUD;
			h->num = 4;
			Pawn* min = NULL;
			int minn = 0;
			for(vector<Block*>::iterator i = blocks.begin(); i != blocks.end(); i++)
			{
				if((*i)->value > minn && GGame.players[(*i)->pattachment] != GGame.huds[0]->player && GGame.players[(*i)->pattachment] != GGame.huds[1]->player && GGame.players[(*i)->pattachment] != GGame.huds[2]->player)
				{
					minn = (*i)->value;
					min = GGame.players[(*i)->pattachment];
				}
			}
			delete textbox;
			msgtext.str("");
			msgtext << "And finally player " << min->pnum << ".";
			textbox = new SubStateText(msgtext.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			h->player = min;
			counting = false;
			progress = 0;
			GGame.huds.push_back(h);
			++ndone2;
		}
	}
	else if(textbox->Tick(dtime) == S_Next)
		counting = true;
	
	return S_Continue;
}

void TurnOrderFinder::Draw(BITMAP* dest)
{
	GGame.board->Draw(dest, camx, camy, 0, 0, dest->w, dest->h);
	for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++)
	{
		int x, y;
		GGame.board->RealToScreen(camx, camy, dest->w, dest->h, (*i)->x, (*i)->y, x, y);
		(*i)->Draw(dest, x, y, 64, 64);
	}
	for(vector<Block*>::iterator i = blocks.begin(); i != blocks.end(); i++)
	{
		int x, y;
		GGame.board->RealToScreen(camx, camy, dest->w, dest->h, GGame.players[(*i)->pattachment]->x - 16, GGame.players[(*i)->pattachment]->y - 128 - 64, x, y);
		(*i)->Draw(dest, x, y, 32, 32);
	}
	for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
	{
		int x, y;
		if((*i)->num == 1)
		{
			x = y = 0;
		}
		else if((*i)->num == 2)
		{
			x = SCREEN_W - 96;
			y = 0;
		}
		else if((*i)->num == 3)
		{
			x = 0;
			y = SCREEN_H - 96;
		}
		else
		{
			x = SCREEN_W - 96;
			y = SCREEN_H - 96;
		}
		(*i)->Draw(dest, x, y, 96, 96);
	}

	GGame.host->DrawPortrait(dest, SCREEN_W - 10 - 128, 100, 128, 128);
	
	if(textbox) textbox->Draw(dest);
}

bool MainGame::InitLogic()
{
	camx = GGame.huds[0]->player->x;
	camy = GGame.huds[0]->player->y;
	GGame.turn = 1;
	return true;
}

bool MainGame::InitGraphics()
{
	return true;
}

void MainGame::EnterState(EState news)
{
	state = news;
	progress = 0;
	switch(state)
	{
		case PreRoll:
			block.pattachment = curturn;
			break;
		case Rolling:
			block.rolling = true;
			GGame.huds[curturn]->player->GotoState(PS_Rolling);
			break;
		case Wait:
			movesleft = block.value;
			break;
		case TrackingPlayer:
		{
			int x, y;
			GGame.board->GetLocationOfSquare(GGame.huds[curturn]->player->location + 1, x, y);
			GGame.huds[curturn]->player->MoveTo(x, y, .5);
			break;
		}
	}
}

EStatus MainGame::Tick(double dtime)
{
	if(paused)
	{
		if(keypressed())
		{
			int key = readkey();
			if(key >> 8 == KEY_ESC)
			{
				RequestState(Menu::Factory);
				return S_Next;
			}
			else
				paused = false;
		}
		return S_Continue;
	}
	
	progress += dtime;
	
	if(keypressed())
	{
		int key = readkey();
		if(key >> 8 == KEY_ESC)
			paused = true;
		else
			simulate_keypress(key);
	}
	
	camx = GGame.huds[curturn]->player->x;
	camy = GGame.huds[curturn]->player->y;
	GGame.board->Tick(dtime);
	block.Tick(dtime);
	for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
		(*i)->Tick(dtime);
	for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++)
	{
		EStatus s = (*i)->Tick(dtime);
		if(state == Rolling && s == S_Notice)
		{
			block.Hit();
			EnterState(Wait);
		}
	}
	if(state == PreRoll && progress > 1)
		EnterState(Rolling);
	if(state == Wait && progress >= 1)
		EnterState(TrackingPlayer);
	if(state == TrackingPlayer)
	{
		
		if(GGame.huds[curturn]->player->dx == GGame.huds[curturn]->player->x && GGame.huds[curturn]->player->dy == GGame.huds[curturn]->player->y)
		{
			GGame.huds[curturn]->player->location++;
			--movesleft;
			if(movesleft == 0)
				EnterState(Wait2);
			else
			{
				int x, y;
				GGame.board->GetLocationOfSquare(GGame.huds[curturn]->player->location + 1, x, y);
				GGame.huds[curturn]->player->MoveTo(x, y, .5);
			}
		}
	}
	if(state == Wait2 && progress >= .5)
	{
		++curturn;
		if(curturn >= GGame.huds.size())
		{
			++GGame.turn;
			if(GGame.turn > GGame.nturns)
			{
				RequestState(EndGame::Factory);
				return S_Next;
			}
			curturn = 0;
		}
		EnterState(PreRoll);
	}
	
	return S_Continue;
}

void MainGame::Draw(BITMAP* dest)
{
	int x, y;
	GGame.board->Draw(dest, camx, camy, 0, 0, dest->w, dest->h);
	
	for(int i = GGame.huds.size() - 1; i > curturn; i--)
	{
		GGame.board->RealToScreen(camx, camy, dest->w, dest->h, GGame.huds[i]->player->x, GGame.huds[i]->player->y, x, y);
		GGame.huds[i]->player->Draw(dest, x, y, 64, 64);
	}
	for(int i = 0; i <= curturn; i++)
	{
		GGame.board->RealToScreen(camx, camy, dest->w, dest->h, GGame.huds[i]->player->x, GGame.huds[i]->player->y, x, y);
		GGame.huds[i]->player->Draw(dest, x, y, 64, 64);
	}
	
	if(state == PreRoll)
		GGame.huds[curturn]->player->instance->DrawPortrait(dest, (dest->w / 2) - 64, (dest->h / 2) - 64, 128, 128);
	if(state == Rolling || state == Wait)
	{
		GGame.board->RealToScreen(camx, camy, dest->w, dest->h, GGame.huds[curturn]->player->x - 16, GGame.huds[curturn]->player->y - 128 - 64, x, y);
		block.Draw(dest, x, y, 32, 32);
	}
	if(state == TrackingPlayer || state == Wait)
		textprintf_centre(dest, font, dest->w / 2, (text_height(font) + 2) * 2, 0, "Moves left: %d", movesleft);
	textout_centre(dest, font, GGame.huds[curturn]->player->instance->name, dest->w / 2, text_height(font) + 2, 0);
	textprintf_centre(dest, font, dest->w / 2, 0, 0, "Turn: %d / %d", GGame.turn, GGame.nturns);
	for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
	{
		int x, y;
		if((*i)->num == 1)
		{
			x = y = 0;
		}
		else if((*i)->num == 2)
		{
			x = SCREEN_W - 96;
			y = 0;
		}
		else if((*i)->num == 3)
		{
			x = 0;
			y = SCREEN_H - 96;
		}
		else
		{
			x = SCREEN_W - 96;
			y = SCREEN_H - 96;
		}
		(*i)->Draw(dest, x, y, 96, 96);
	}
	
	if(paused)
	{
		textout_centre(dest, font, "PAUSED", dest->w / 2, dest->h - (text_height(font) + 2) * 3, 0);
		textout_centre(dest, font, "ESC Quits", dest->w / 2, dest->h - (text_height(font) + 2) * 2, 0);
		textout_centre(dest, font, "Else resumes", dest->w / 2, dest->h - (text_height(font) + 2) * 1, 0);
	}
}

bool EndGame::InitLogic()
{
	EnterState(Suspense);
	return true;
}

bool EndGame::InitGraphics()
{
	int p = -2;
	for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++, p++)
	{
		(*i)->MoveTo(p * 94, 150, 3);
	}
	
	p1 = -1;
	int minn = -1;
	for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
	{
		if((*i)->player->location > minn)
		{
			minn = (*i)->player->location;
			p1 = (*i)->num;
		}
	}
	p2 = -1;
	minn = -1;
	for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
	{
		if((*i)->player->location > minn && (*i)->num != p1)
		{
			minn = (*i)->player->location;
			p2 = (*i)->num;
		}
	}
	p3 = -1;
	minn = -1;
	for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
	{
		if((*i)->player->location > minn && (*i)->num != p1 && (*i)->num != p2)
		{
			minn = (*i)->player->location;
			p3 = (*i)->num;
		}
	}
	p4 = -1;
	minn = -1;
	for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
	{
		if((*i)->player->location > minn && (*i)->num != p1 && (*i)->num != p2 && (*i)->num != p3)
		{
			minn = (*i)->player->location;
			p4 = (*i)->num;
		}
	}
	return true;
}

void EndGame::EnterState(EState news)
{
	stringstream msg;
	state = news;
	progress = 0;
	counting = false;
	delete textbox;
	textbox = NULL;
	switch(state)
	{
		case Suspense:
			textbox = new SubStateText("Well, that's the end of the game. Let's see how our contestants did!", 10, 100, SCREEN_W - 158, 128, TM_Filled);
			break;
		case Announce4:
			msg << "Coming in fourth place is... " << GGame.huds[p4 - 1]->player->instance->name << "!";
			GGame.huds[p4 - 1]->player->instance->Animate(S_Jumping, 1);
			textbox = new SubStateText(msg.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			break;
		case Announce3:
			msg << "And in third place... " << GGame.huds[p3 - 1]->player->instance->name << "!";
			GGame.huds[p3 - 1]->player->instance->Animate(S_Jumping, 1);
			textbox = new SubStateText(msg.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			break;
		case Announce2:
			msg << "Second place was... " << GGame.huds[p2 - 1]->player->instance->name << "!";
			GGame.huds[p2 - 1]->player->instance->Animate(S_Jumping, 1);
			textbox = new SubStateText(msg.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			break;
		case Announce1:
			msg << "So therefore the winner is... " << GGame.huds[p1 - 1]->player->instance->name << "!";
			GGame.huds[p1 - 1]->player->instance->Animate(S_Jumping, 1);
			textbox = new SubStateText(msg.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			break;
		case GoodGame:
			msg << "Well, that was a good game. Hope you enjoyed it!";
			textbox = new SubStateText(msg.str().c_str(), 10, 100, SCREEN_W - 158, 128, TM_Filled);
			break;
	}
}

EStatus EndGame::Tick(double dtime)
{
	if(counting)
		progress += dtime;
	if(textbox && textbox->Tick(dtime) == S_Next)
		counting = true;
	
	if(progress >= .5)
	{
		if(state == GoodGame)
		{
			set_config_int("tinparty", "games", get_config_int("tinparty", "games", 0) + 1);
			RequestState(Menu::Factory);
			return S_Next;
		}
		EnterState((EState) (state + 1));
	}
	
	for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++)
		(*i)->Tick(dtime);
	
	return S_Continue;
}

void EndGame::Draw(BITMAP* dest)
{
	GGame.board->Draw(dest, 0, 0, 0, 0, dest->w, dest->h);
	for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++)
	{
		int x, y;
		GGame.board->RealToScreen(0, 0, dest->w, dest->h, (*i)->x, (*i)->y, x, y);
		(*i)->Draw(dest, x, y, 64, 64);
	}
	for(vector<HUD*>::iterator i = GGame.huds.begin(); i != GGame.huds.end(); i++)
	{
		int x, y;
		if((*i)->num == 1)
		{
			x = y = 0;
		}
		else if((*i)->num == 2)
		{
			x = SCREEN_W - 96;
			y = 0;
		}
		else if((*i)->num == 3)
		{
			x = 0;
			y = SCREEN_H - 96;
		}
		else
		{
			x = SCREEN_W - 96;
			y = SCREEN_H - 96;
		}
		(*i)->Draw(dest, x, y, 96, 96);
	}

	GGame.host->DrawPortrait(dest, SCREEN_W - 10 - 128, 100, 128, 128);
	
	if(textbox) textbox->Draw(dest);
}

// The end
