// pawn.cpp
//  Player Pawns

#include "tinparty.h"

using namespace std;

Pawn::~Pawn()
{
}

void Pawn::MoveTo(int x, int y, double timetoarrive)
{
	sx = this->x;
	sy = this->y;
	dx = x;
	dy = y;
	progress = 0;
	timedialation = 1 / timetoarrive;
	if(instance) instance->Animate(S_Walking, 1);
}

EStatus Pawn::Tick(double dtime)
{
	EStatus s = S_Continue;
	
	if(x != dx || y != dy)
	{
		x = sx + (int) ((dx - sx) * progress);
		y = sy + (int) ((dy - sy) * progress);
		progress += dtime * timedialation;
		if(progress >= 1)
		{
			sx = x = dx;
			sy = y = dy;
			s = S_Next;
			if(instance) instance->Animate(S_Standing, 1);
		}
	}
	
	if(instance && instance->Tick(dtime) == S_Next) s = S_Next;
	
	switch(state)
	{
		case PS_RollingInAir:
			if(instance && instance->progress >= .5)
			{
				s = S_Notice;
				GotoState(PS_None);
			}
			break;
	}
	
	return s;
}

void Pawn::Draw(BITMAP* dest, int x, int y, int w, int h)
{
	if(instance) instance->DrawSelf(dest, x - (w / 2), y - h, w, h);
}

void HumanPawn::GotoState(EPawnState news)
{
	Pawn::GotoState(news);
	clear_keybuf();
}

EStatus HumanPawn::Tick(double dtime)
{
	switch(state)
	{
		case PS_Rolling:
			if(keypressed() && instance && instance->state == S_Standing)
			{
				int key = readkey();
				if(key >> 8 == KEY_SPACE)
				{
					instance->Animate(S_Jumping, 1);
					state = PS_RollingInAir;
				}
			}
			break;
	}
	return Pawn::Tick(dtime);
}

void ComputerPawn::GotoState(EPawnState news)
{
	Pawn::GotoState(news);
	progress = 0;
	timetoreact = (double) rand() / RAND_MAX * 3;
}

EStatus ComputerPawn::Tick(double dtime)
{
	progress += dtime;
	switch(state)
	{
		case PS_Rolling:
			if(progress >= timetoreact && instance && instance->state == S_Standing)
			{
				instance->Animate(S_Jumping, 1);
				state = PS_RollingInAir;
			}
			if(instance && instance->state != S_Standing)
				progress = 0;
	}
	return Pawn::Tick(dtime);
}

// The end
