// plat.cpp
//  Platform compatibility layer

#include "tinparty.h"

using namespace std;

#ifdef __linux__
void* GetDiff(void* _then, double& dtime)
{
	timeval* then = (timeval*) _then;
	if(then == NULL)
	{
		dtime = 0;
		timeval* now = new timeval;
		gettimeofday(now, NULL);
		return now;
	}
	timeval* now = new timeval;
	gettimeofday(now, NULL);
	timeval diff;
	
	diff.tv_sec = now->tv_sec - then->tv_sec;
	diff.tv_usec = now->tv_usec - then->tv_usec;
	if(now->tv_usec < then->tv_usec)
		diff.tv_usec = 100000 + now->tv_usec - then->tv_usec;
		
	dtime = diff.tv_sec;
	dtime += (double) diff.tv_usec / 1e6;
	
	delete then;
	return (void*) now;
}

void Sleep(uint32_t msecs)
{
	timeval t;
	t.tv_sec = msecs / 1000;
	t.tv_usec = 1000 * (msecs % 1000);
	select(0, NULL, NULL, NULL, &t);
}
#endif

#if __MINGW32__ || _MSC_VER
void* GetDiff(void* _then, double& dtime)
{
	LARGE_INTEGER* then = (LARGE_INTEGER*) _then;
	if(then == NULL)
	{
		dtime = 0;
		LARGE_INTEGER* now = new LARGE_INTEGER;
		QueryPerformanceCounter(now);
		return now;
	}
	LARGE_INTEGER* now = new LARGE_INTEGER;
	QueryPerformanceCounter(now);
	LARGE_INTEGER diff;
 	diff.QuadPart = now->QuadPart - then->QuadPart;
	LARGE_INTEGER pers;
	QueryPerformanceFrequency(&pers);
	dtime = (double) diff.QuadPart / pers.QuadPart;
	delete then;
	return (void*) now;
}

void Sleep(uint32_t milliseconds)
{
	Sleep((unsigned long) milliseconds);
}
#endif

// The end
