// pregame.h
//  PreGame GameState

#include "tinparty.h"

using namespace std;

SubStateCharSelect::SubStateCharSelect(): curplayerselection(0), caret_position(0)
{
}

EStatus SubStateCharSelect::Tick(double dtime)
{
	if(keypressed())
	{
		int key = readkey();
		if(key >> 8 == KEY_LEFT)
		{
			//SOUND menu move
			if(caret_position == 0)
				caret_position = Character::chars.size();
			--caret_position;
		}
		else if(key >> 8 == KEY_RIGHT)
		{
			//SOUND menu move
			++caret_position;
			if(caret_position >= Character::chars.size())
				caret_position = 0;
		}
		if(key >> 8 == KEY_UP)
		{
			//SOUND menu move
			if(caret_position >= PORTRAITS_PER_ROW)
				caret_position -= PORTRAITS_PER_ROW;
		}
		else if(key >> 8 == KEY_DOWN)
		{
			//SOUND menu move
			if(caret_position < Character::chars.size() - PORTRAITS_PER_ROW)
				caret_position += PORTRAITS_PER_ROW;
		}
		else if(key >> 8 == KEY_SPACE || key >> 8 == KEY_ENTER)
		{
			bool good = true;
			for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++)
			{
				if((*i)->instance && (*i)->instance->id == caret_position)
				{
					good = false;
					//SOUND menu back
					break;
				}
			}
			if(!Character::chars[caret_position]->playable)
			{
				if(get_config_int("tinparty", "games", 0) < 12)
					good = false;
			}
			if(good)
			{
				GGame.players[curplayerselection]->instance = Character::chars[caret_position];
				curplayerselection++;
			}
			if(curplayerselection >= GGame.players.size())
				return S_Next;
		}
		else
			simulate_keypress(key);
	}
	
	return S_Continue;
}

void SubStateCharSelect::Draw(BITMAP* dest)
{
	uint32_t p = 0;
	for(vector<Character*>::iterator i = Character::chars.begin(); i != Character::chars.end(); i++, p++)
		(*i)->DrawPortrait(dest, 50 + (138 * (p % PORTRAITS_PER_ROW)), 50 + (138 * (p / PORTRAITS_PER_ROW)), 128, 128);
	for(vector<Pawn*>::iterator i = GGame.players.begin(); i != GGame.players.end(); i++)
	{
		if((*i)->instance)
		{
			draw_sprite(dest, caret, 50 + (138 * ((*i)->instance->id % PORTRAITS_PER_ROW)), 50 + (138 * ((*i)->instance->id / PORTRAITS_PER_ROW)));
			textprintf_centre(dest, font, 50 + (138 * ((*i)->instance->id % PORTRAITS_PER_ROW)) + (caret->w / 2), 50 + (138 * ((*i)->instance->id / PORTRAITS_PER_ROW)) + (caret->h / 2) - (text_height(font) / 2), 0, "%d", (*i)->pnum);
		}
	}
	DrawCaret(50 + (138 * (caret_position % PORTRAITS_PER_ROW)), 50 + (138 * (caret_position / PORTRAITS_PER_ROW)));
}

SubStateBoardSelect::SubStateBoardSelect(): caret_position(0)
{
}

EStatus SubStateBoardSelect::Tick(double dtime)
{
	if(keypressed())
	{
		int key = readkey();
		if(key >> 8 == KEY_LEFT)
		{
			//SOUND menu move
			if(caret_position == 0)
				caret_position = Board::boards.size();
			--caret_position;
		}
		else if(key >> 8 == KEY_RIGHT)
		{
			//SOUND menu move
			++caret_position;
			if(caret_position >= Board::boards.size())
				caret_position = 0;
		}
		if(key >> 8 == KEY_UP)
		{
			//SOUND menu move
			if(caret_position >= PORTRAITS_PER_ROW)
				caret_position -= PORTRAITS_PER_ROW;
		}
		else if(key >> 8 == KEY_DOWN)
		{
			//SOUND menu move
			if(caret_position < Board::boards.size() - PORTRAITS_PER_ROW)
				caret_position += PORTRAITS_PER_ROW;
		}
		else if(key >> 8 == KEY_SPACE || key >> 8 == KEY_ENTER)
		{
			GGame.board = Board::boards[caret_position];
			return S_Next;
		}
		else
			simulate_keypress(key);
	}
	
	return S_Continue;
}

void SubStateBoardSelect::Draw(BITMAP* dest)
{
	uint32_t p = 0;
	for(vector<Board*>::iterator i = Board::boards.begin(); i != Board::boards.end(); i++, p++)
		(*i)->DrawPortrait(dest, 50 + (138 * (p % PORTRAITS_PER_ROW)), 50 + (138 * (p / PORTRAITS_PER_ROW)), 128, 128);
	DrawCaret(50 + (138 * (caret_position % PORTRAITS_PER_ROW)), 50 + (138 * (caret_position / PORTRAITS_PER_ROW)));
}

PreGame::~PreGame()
{
	destroy_bitmap(fade_gradient);
	fade_gradient = NULL;
	delete textbox;
	textbox = NULL;
}

void PreGame::EnterState(bool backward)
{
	delete textbox;
	textbox = NULL;
	caret_position = 0;
	switch(curscreen)
	{
		case NumberScreen:
			if(backward)
			{
				if(GGame.type == GT_FreeForAll || GGame.type == GT_Teams || GGame.type == GT_OddManOut)
					msgtext = "How many people are going to be playing?";
			}
			else
			{
				msgtext = "Welcome to Tin Party! I'm $ff0000";
				msgtext += GGame.host->name;
				msgtext += "$aa0000, and I'll be your host today.";
				if(GGame.type == GT_FreeForAll || GGame.type == GT_Teams || GGame.type == GT_OddManOut)
					msgtext += " Now we need to find out how many people are going to be playing.";
			}
			textbox = new SubStateText(msgtext.c_str(), 128 + 10, SCREEN_H - 128 - 10, SCREEN_W - (2 * (128 + 10)), 128);
			break;
		case CharSelectScreen:
			msgtext = "Now select the characters for the game.";
			textbox = new SubStateText(msgtext.c_str(), 128 + 10, SCREEN_H - 128 - 10, SCREEN_W - (2 * (128 + 10)), 128);
			charsel = new SubStateCharSelect;
			if(backward)
			{
				charsel->curplayerselection = GGame.players.size() - 1;
				charsel->caret_position = GGame.players[charsel->curplayerselection]->instance->id;
				GGame.players[charsel->curplayerselection]->instance = NULL;
			}
			break;
		case BoardSelectScreen:
			msgtext = "One last thing: Where Do You Want To Go Today?";
			textbox = new SubStateText(msgtext.c_str(), 128 + 10, SCREEN_H - 128 - 10, SCREEN_W - (2 * (128 + 10)), 128);
			boardsel = new SubStateBoardSelect;
			if(backward && GGame.board)
				charsel->caret_position = GGame.board->id;
			break;
	}
}

bool PreGame::InitLogic()
{
	EnterState();
	
	return textbox;
}

bool PreGame::InitGraphics()
{
	fade_gradient = create_bitmap(SCREEN_W, 1);
	if(!fade_gradient)
		cout << "Failed to make a fade bitmap" << endl;
	for(uint32_t i = 0; i < SCREEN_W; i++)
	{
		int r, g, b;
		hsv_to_rgb((double) i / SCREEN_W * 360, .25, 1, &r, &g, &b);
		putpixel(fade_gradient, i, 0, makecol(r, g, b));
	}
	
	return fade_gradient;
}

EStatus PreGame::Tick(double dtime)
{
	bg_phase += (uint32_t) (dtime * SCREEN_W / 5);
	bg_phase = bg_phase % SCREEN_W;

	if(textbox) textbox->Tick(dtime);
	
	switch(curscreen)
	{
		case NumberScreen:
			if(keypressed())
			{
				int key = readkey();
				if(key >> 8 == KEY_ESC)
				{
					RequestState(Menu::Factory);
					//SOUND menu back
					return S_Next;
				}
				if(key >> 8 == KEY_DOWN)
				{
					caret_position++;
					if(caret_position > 4) caret_position = 0;
					//SOUND menu move
				}
				if(key >> 8 == KEY_UP)
				{
					if(caret_position <= 0) caret_position = 5;
					caret_position--;
					//SOUND menu move
				}
				if(key >> 8 == KEY_SPACE || key >> 8 == KEY_ENTER)
				{
					//SOUND menu forward
					switch(caret_position)
					{
						case 0:
						{
							Pawn* p;
							p = new HumanPawn(1);
							GGame.players.push_back(p);
							p = new ComputerPawn(2);
							GGame.players.push_back(p);
							p = new ComputerPawn(3);
							GGame.players.push_back(p);
							p = new ComputerPawn(4);
							GGame.players.push_back(p);
							break;
						}
						case 1:
						{
							Pawn* p;
							p = new HumanPawn(1);
							GGame.players.push_back(p);
							p = new HumanPawn(2);
							GGame.players.push_back(p);
							p = new ComputerPawn(3);
							GGame.players.push_back(p);
							p = new ComputerPawn(4);
							GGame.players.push_back(p);
							break;
						}
						case 2:
						{
							Pawn* p;
							p = new HumanPawn(1);
							GGame.players.push_back(p);
							p = new HumanPawn(2);
							GGame.players.push_back(p);
							p = new HumanPawn(3);
							GGame.players.push_back(p);
							p = new ComputerPawn(4);
							GGame.players.push_back(p);
							break;
						}
						case 3:
						{
							Pawn* p;
							p = new HumanPawn(1);
							GGame.players.push_back(p);
							p = new HumanPawn(2);
							GGame.players.push_back(p);
							p = new HumanPawn(3);
							GGame.players.push_back(p);
							p = new HumanPawn(4);
							GGame.players.push_back(p);
							break;
						}
						case 4:
							RequestState(Menu::Factory);
							return S_Next;
					}
					curscreen = CharSelectScreen;
					EnterState();
				}
			}
			break;
		case CharSelectScreen:
			if(charsel->Tick(dtime) == S_Next)
			{
				curscreen = BoardSelectScreen;
				EnterState();
			}
			if(keypressed())
			{
				int key = readkey();
				if(key >> 8 == KEY_ESC)
				{
					//SOUND menu back
					if(charsel->curplayerselection == 0)
					{
						delete charsel;
						curscreen = NumberScreen;
						EnterState(true);
					}
					else
					{
						charsel->curplayerselection--;
						charsel->caret_position = GGame.players[charsel->curplayerselection]->instance->id;
						GGame.players[charsel->curplayerselection]->instance = NULL;
					}
				}
			}
			break;
		case BoardSelectScreen:
			if(boardsel->Tick(dtime) == S_Next)
			{
				RequestState(TurnOrderFinder::Factory);
				//SOUND game start
				return S_Next;
			}
			if(keypressed())
			{
				int key = readkey();
				if(key >> 8 == KEY_ESC)
				{
					//SOUND menu back
					delete boardsel;
					curscreen = CharSelectScreen;
					EnterState(true);
				}
			}
			break;
	}
	
	return S_Continue;
}

void PreGame::Draw(BITMAP* dest)
{
	stretch_blit(fade_gradient, dest, 0, 0, fade_gradient->w, fade_gradient->h, bg_phase, 0, dest->w, 1);
	stretch_blit(fade_gradient, dest, 0, 0, fade_gradient->w, fade_gradient->h, bg_phase - fade_gradient->w, 0, dest->w, 1);
	for(uint32_t i = 0; i < (uint32_t) (log((double) SCREEN_H) / log(2.)) + 1; i++)
		blit(dest, dest, 0, 0, 0, 1 << i, SCREEN_W, 1 << i);

	GGame.host->DrawPortrait(dest, SCREEN_W - 10 - 128, SCREEN_H - 10 - 128, 128, 128);
	
	if(textbox) textbox->Draw(dest);
	
	switch(curscreen)
	{
		case NumberScreen:
			textprintf_centre(dest, font, SCREEN_W / 2, SCREEN_H / 3, 0,
				"1 human, 3 computer");
			textprintf_centre(dest, font, SCREEN_W / 2, (SCREEN_H / 3) + caret_height / 2, 0,
				"2 human, 2 computer");
			textprintf_centre(dest, font, SCREEN_W / 2, (SCREEN_H / 3) + (2 * caret_height / 2), 0,
				"3 human, 1 computer");
			textprintf_centre(dest, font, SCREEN_W / 2, (SCREEN_H / 3) + (3 * caret_height / 2), 0,
				"4 human, 0 computer");
			textprintf_centre(dest, font, SCREEN_W / 2, (SCREEN_H / 3) + (4 * caret_height / 2), 0,
				"Back");
			DrawCaret(SCREEN_W / 2 - 120, (SCREEN_H / 3) + (caret_position * caret_height / 2) - (caret_height / 2));
			break;
		case CharSelectScreen:
			charsel->Draw(dest);
			break;
		case BoardSelectScreen:
			boardsel->Draw(dest);
			break;
	}
}

// The end
