// tinparty.cpp
//  Description

#include "tinparty.h"

using namespace std;

GameInfo GGame;
uint32_t caret_width = 0;
uint32_t caret_height = 0;
BITMAP* caret = NULL;

static GameState* curstate = NULL;
static GameState* (*nextstate)() = Menu::Factory;
static bool quit = false;
static uint32_t caret_rotation = 0;
static double caret_phase = 0;
static BITMAP* page1 = NULL;
static BITMAP* page2 = NULL;
static bool use2pages = true;
static bool dbuf = false;
static uint32_t page = 0;

void NewGame()
{
	GGame.turn = 0;
	GGame.nturns = 10;
	GGame.players.clear();
	GGame.curplayerturn = 0;
	GGame.host = Character::GetByName("Troll");
	if(!GGame.host)
		cout << "Can't get host by name" << endl;
}

void RunState()
{
	if(!curstate)
	{
		quit = true;
		return;
	} 
	
	double dtime = 0;
	void* now = GetDiff(NULL, dtime);
	
	curstate->Tick(dtime);
	while(1)
	{
		now = GetDiff(now, dtime);
		if(dtime < MIN_DTIME)
		{
			Sleep((uint32_t) ((MIN_DTIME - dtime) * 1000));
			now = GetDiff(now, dtime);
		}
		caret_rotation += (uint32_t) (dtime * 128);
		caret_rotation = caret_rotation & 0xff;
		caret_phase += dtime * 2;
		caret_phase = fmod(caret_phase, 6.28);
		switch(curstate->Tick(dtime))
		{
			case S_Quit: quit = true; return;
			case S_Next: return;
		}
		BITMAP* p = page2;dbuf ? page2 : use2pages ? !page ? page1 : page2 : page1;
		page = !page;
		curstate->Draw(p);
		if(dbuf)
      		blit(page2, page1, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		else show_video_bitmap(p);
	}
}

void RequestState(GameState* (*factory)())
{ nextstate = factory; }

void DrawCaret(uint32_t x, uint32_t y)
{
	rotate_sprite(dbuf ? page2 : use2pages ? !page ? page1 : page2 : page1, caret, x + (caret_width / 5), y + (caret_height / 4) + (uint32_t) (sin(caret_phase) * caret_height / 8), itofix(caret_rotation));
}

static void GotoState()
{
	if(!nextstate) return;
	GameState* s = (*nextstate)();
	nextstate = NULL;
	if(s->InitLogic() && s->InitGraphics())
	{
		if(curstate) curstate->Destroy();
		curstate = s;
	}
	clear_keybuf();
}

int main(int argc, char** argv[])
{
	allegro_init();
	set_config_file("data/tinparty.ini");
	install_keyboard();
	set_color_depth(32);
	set_gfx_mode(GFX_AUTODETECT, 800, 600, 0, 0);
	//set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0);
	text_mode(-1);
	set_display_switch_mode(SWITCH_BACKGROUND);
	srand(time(NULL));
	
	page1 = create_video_bitmap(SCREEN_W, SCREEN_H);
	if(!page1) return 1;
	show_video_bitmap(page1);
	//page2 = create_video_bitmap(SCREEN_W, SCREEN_H);
	if(!page2)
	{
		cout << "Can't get two pages" << endl;
		use2pages = false;
    	page2 = create_system_bitmap(SCREEN_W, SCREEN_H);
    	if(page2) dbuf = true;
    	else
    		cout << "Can't get a system bitmap" << endl;
	}
	
	Character::StaticInitGraphics();
	Board::StaticInitGraphics();
	
	caret = load_bitmap("data/caret.tga", NULL);
	if(!caret)
		cout << "Failed to load data/caret.tga" << endl;
	caret_width = (uint32_t) sqrt((double) caret->w * caret->w + caret->h * caret->h);
	caret_height = caret->h * 2;

	while(!quit)
	{
		if(nextstate)
			GotoState();
		RunState();
	}
	
	destroy_bitmap(page1);
	destroy_bitmap(page2);
	
	destroy_bitmap(caret);
	caret = NULL;
	
	Board::StaticDestroy();
	Character::StaticDestroy();
	
	return 0;
}
END_OF_MAIN()

// The end
