// tinparty.h
//  Main include file for Tin Party

#ifndef TINPARTY_H
#define TINPARTY_H

// Global includes
#include <allegro.h>
#include <iostream>
#include <vector>
#include <sstream>
#include <math.h>

// Platform compatibility
#include "plat.h"

/// Minimum dtime in seconds
#define MIN_DTIME ((double) 60 / 1000)
/// Portraits per row
#define PORTRAITS_PER_ROW 5

/// Return codes for states
enum EStatus
{
	S_Continue,		// Continue normally
	S_Quit,			// Quit the game
	S_Next,			// Switch to a new state
	S_Notice,		// Something happened!
};

/// Possible gametypes
enum EGameTypes
{
	GT_FreeForAll,
	GT_Teams,
	GT_OddManOut,
	GT_OneOnOne,
	GT_SinglePlayer,
};

/// Represents unlockable content
enum EUnlockable
{
	E_Unlockable
};

// Forward declarations
class	GameInfo;
class	Character;
class	Block;
class	HUD;
class	GameState;
class		SubStateText;
class		Menu;
class		PreGame;
class		TurnOrderFinder;
class		Pawn;
class			HumanPawn;

// Global variables
/// The current running game
extern GameInfo GGame;
/// Width of the caret, for DRSes
extern uint32_t caret_width;
/// Height of the caret, for DRSes
extern uint32_t caret_height;
/// Caret bitmap
extern BITMAP* caret;

void NewGame();
EStatus RunState(GameState* s);
void RequestState(GameState* (*factory)());
void en_renderf(BITMAP* dest, const FONT* font, int x, int y, const char* msg, ...);
void DrawCaret(uint32_t x, uint32_t y);

#include "character.h"	// Characters
#include "board.h"		// Boards
#include "gameinfo.h"	// Game status information
#include "gamestate.h"	// Base class for Game States
#include "menu.h"		// Menu GameState
#include "pregame.h"	// PreGame GameState
#include "game.h"	// Game GameState
#include "pawn.h"		// Player Pawns

#endif

// The end
